from .sentences import *
from .quotes import *
from .english_to_cocanb import toc
from .cocanb_to_english import toe

def english_to_cocanb(initial):
    q_initial = split_quotes(initial)
    sentences = split_sentences(q_initial["outside"])
    for sentence in sentences:
        sentence.string = toc(sentence.string)
    for quote in q_initial["inside"]: 
        q_sentences = split_sentences(quote)
        for sentence in q_sentences:
            sentence.string = toc(sentence.string)
        q_initial["inside"][q_initial["inside"].index(quote)] = join_sentences(q_sentences)
    joined = join_sentences(sentences)
    final = join_quotes({
        "outside": joined,
        "inside": q_initial["inside"]
    })
    return final

def cocanb_to_english(initial):
    q_initial = split_quotes(initial)
    sentences = split_sentences(q_initial["outside"])
    for sentence in sentences:
        sentence.string = toe(sentence.string)
    for quote in q_initial["inside"]: 
        q_sentences = split_sentences(quote)
        for sentence in q_sentences:
            sentence.string = toe(sentence.string)
        q_initial["inside"][q_initial["inside"].index(quote)] = join_sentences(q_sentences)
    joined = join_sentences(sentences)
    final = join_quotes({
        "outside": joined,
        "inside": q_initial["inside"]
    })
    return final


def split_quotes(initial):
    _outside = "<quote>".join(initial.split("\"")[::2])
    _inside = initial.split("\"")[1::2]
    return {"outside": _outside, "inside": _inside}

def join_quotes(initial):
    final = initial["outside"]
    for quote in initial["inside"]:
        final = final.replace("<quote>", f"\"{quote}\"", 1)
    return final


from re import sub, findall
from numpy.random import choice

a = ["a","à","á","â","ä"]
a_p = [8/15, 2/15, 2/15, 2/15, 1/15]
e = ["e","è","é","ê"]
e_p = [8/14, 2/14, 2/14, 2/14]
i = ["i","î","í","ì"]
i_p = [8/14, 2/14, 2/14, 2/14]
o = ["o","ô","ó","ò","ö"]
o_p = [8/15, 2/15, 2/15, 2/15, 1/15]
u = ["u","ü","ù","ú","û"]
u_p = [8/15, 2/15, 2/15, 2/15, 1/15]
y = ["y","ý"]
c = ["c","č","ć"]
r = ["r","ř"]
s = ["s","š","ś"]
n = ["n","ň","ń"]
d = ["d","đ"]
l = ["l","ł"]
g = ["g","ğ"]
z = ["z","ž","ź"]

# I just picked some keys from my keyboard. This translator should remove everything here before getting to this point
cocanb_char = "|"
cocan_char = "&"
non_char = "¦"
no_char = "¬"
special_char = "¤"

def add_diacritics(initial):
    initial = sub("[nňń][oóòôö]$", no_char, sub("[nňń][oóòôö][nňń]", non_char, sub("[cćč][oóòôö][cćč][aáàâä]$", cocan_char, sub("[cćč][oóòôö][cćč][aáàâä][nňń]", cocanb_char, initial))))
    special = findall("<.*>", initial)
    initial = sub("<.*>", special_char, initial)
    final = []
    for letter in initial:
        if letter == "a":
            final.append(choice(a, p=a_p))
        elif letter == "e":
            final.append(choice(e, p=e_p))
        elif letter == "i":
            final.append(choice(i, p=i_p))
        elif letter == "o":
            final.append(choice(o, p=o_p))
        elif letter == "u":
            final.append(choice(u, p=u_p))
        elif letter == "y":
            final.append(choice(y))
        elif letter == "c":
            final.append(choice(c))
        elif letter == "r":
            final.append(choice(r))
        elif letter == "s":
            final.append(choice(s))
        elif letter == "n":
            final.append(choice(n))
        elif letter == "d":
            final.append(choice(d))
        elif letter == "l":
            final.append(choice(l))
        elif letter == "g":
            final.append(choice(g))
        elif letter == "z":
            final.append(choice(z))
        else:
            final.append(letter)
    output = "".join(final)
    output = output.replace(cocanb_char, "Cocán").replace(cocan_char, "Cocá").replace(non_char, choice(["nön", "nôn", "nón", "nòn"])).replace(no_char, choice(["nö", "nô", "nó", "nò"]))
    for sp in special:
        output = output.replace(special_char, sp)
    return output


from unidecode import unidecode

#I wrote this in the meanwhile
def toe(sentence):
    sentence = sentence.replace(" ", "").replace("æ", "ae").replace("œ", "oe")
    initial = sentence[:sentence.index("non")]
    additional = sentence[sentence.index("non")+3:]
    final = []
    letters_gone = 0
    for i in range(int(len(additional)/2)):
        final.append(initial[letters_gone:letters_gone + ord(additional[2*i+1])-97]+additional[2*i])
        letters_gone += ord(additional[2*i+1])-97
    return " ".join(final)


# by nonka

# original just in case i fuck up
# def toc(sentence):
#     initial = sentence.split()
#     additional = ""
#     for i in initial:
#         additional += i[-1] + chr(ord('`') + len(i))
#         initial[initial.index(i)] = i[:-1]
#     final = "".join(initial) + "non" + additional
#     return final

from .spaces import add_spaces
from .diacritics import add_diacritics
from re import sub

def remove_punctuation(sentence):
    return sentence

def toc(sentence):
    initial = remove_punctuation(sentence.split())
    additional = ""
    nums = []
    for i in initial:
        if i.isnumeric():
            nums.append(i)
            initial[initial.index(i)] = "<num>"
    for i in initial:
        if (i[0] != "<" and i[-1] != ">"):
            additional += i[-1] + ("å" * (len(i) // 26)) + chr(ord('`') + (len(i) % 26))
            initial[initial.index(i)] = i[:-1]
    final = (add_diacritics("".join(initial)) + "non" + add_diacritics(additional)).replace("<", " <").replace(">", "> ")
    for num in nums:
        final = final.replace("<num>", num, 1)
    final = add_spaces(final)
    return final


from numpy.random import choice
from re import findall, sub
from unidecode import unidecode

def add_spaces(initial):
    special = findall("<.*>", initial)
    replaced = sub("<.*>", "|", initial)
    replaced = list(replaced)
    for i in replaced:
        replaced[replaced.index(i)] = i + choice(["", " "], p=[0.7, 0.3]) #change
    replaced = "".join(replaced)
    for i in special:
        replaced = replaced.replace("|", i)
    final = sub(" +", " ", replaced)
    return final


import re

class Sentence:
    def __init__(self, string, punc):
        self.string = string
        self.punc = punc
    def join(self):
        return self.string + self.punc

def split_sentences(_input):
    str_list = re.split("[.!\?\n]", _input.strip())
    for i in str_list:
        i = i.strip()
    punc_list = []
    for i in _input:
        if i in [".", "!", "?", "\n"]:
            punc_list.append(i)
    punc_list.append("")
    output = [None] * len(str_list)
    for i in range(len(str_list)):
        output[i] = Sentence(str_list[i], punc_list[i])
    if not output[-1].string:
        output.pop()
    return output

def join_sentences(_input):
    output = ""
    for i in _input:
        output += i.join() + " "
    return output

